@@ -12,10 +12,15 @@ from ._misc import (
 import dask
 import dask.array as da
 from dask.delayed import delayed
-from numpy import empty, float64, zeros
+from numpy import empty, float64, zeros, asarray
 from pandas import DataFrame
 from tqdm import tqdm
 
+try:
+    from functools import lru_cache
+except ImportError:
+    from ._pylru import lrudecorator as lru_cache
+
 from ._ffi import ffi
 from ._ffi.lib import (
     bgen_close,
@@ -141,54 +146,52 @@ def _generate_samples(bgen_file):
     return DataFrame(data=dict(id=["sample_%d" % i for i in range(nsamples)]))
 
 
-class ReadGenotypeVariant(object):
-    def __init__(self, indexing):
-        self._indexing = indexing
+@lru_cache(1)
+def _read_genotype_variant(
+    indexing, nsamples, nalleles, variant_idx, nvariants
+):
 
-    def __call__(self, nsamples, nalleles, variant_idx, nvariants):
+    nalleles = asarray(nalleles, int)
+    ncombss = []
+    variants = []
 
-        ncombss = []
-        variants = []
+    for i in range(variant_idx, variant_idx + nvariants):
+        vg = bgen_open_variant_genotype(indexing[0], i)
 
-        for i in range(variant_idx, variant_idx + nvariants):
-            vg = bgen_open_variant_genotype(self._indexing[0], i)
+        ncombs = bgen_ncombs(vg)
+        ncombss.append(ncombs)
+        g = empty((nsamples, ncombs), dtype=float64)
 
-            ncombs = bgen_ncombs(vg)
-            ncombss.append(ncombs)
-            g = empty((nsamples, ncombs), dtype=float64)
+        pg = ffi.cast("double *", g.ctypes.data)
+        bgen_read_variant_genotype(indexing[0], vg, pg)
 
-            pg = ffi.cast("double *", g.ctypes.data)
-            bgen_read_variant_genotype(self._indexing[0], vg, pg)
+        bgen_close_variant_genotype(indexing[0], vg)
 
-            bgen_close_variant_genotype(self._indexing[0], vg)
+        variants.append(g)
 
-            variants.append(g)
+    G = zeros((nvariants, nsamples, max(ncombss)), dtype=float64)
 
-        G = zeros((nvariants, nsamples, max(ncombss)), dtype=float64)
+    for i in range(0, nvariants):
+        G[i, :, : ncombss[i]] = variants[i]
 
-        for i in range(0, nvariants):
-            G[i, :, : ncombss[i]] = variants[i]
-
-        return G
+    return G
 
 
 def _read_genotype(indexing, nsamples, nvariants, nalleless, size, verbose):
 
     genotype = []
-    rgv = ReadGenotypeVariant(indexing)
 
     c = int((1024 * 1024 * size / 8) // nsamples)
     step = min(c, nvariants)
     tqdm_kwds = dict(desc="Variant mapping", disable=not verbose)
 
+    delcall = delayed(_read_genotype_variant, pure=True, traverse=False)
     for i in tqdm(range(0, nvariants, step), **tqdm_kwds):
         size = min(step, nvariants - i)
-        tup = nsamples, nalleless[i : i + size], i, size
-        delayed_kwds = dict(pure=True, traverse=False)
-        g = delayed(rgv, **delayed_kwds)(*tup)
+        tup = indexing, nsamples, tuple(nalleless[i : i + size]), i, size
         # TODO: THIS IS A HACK
         ncombs = 3
-        g = da.from_delayed(g, (size, nsamples, ncombs), float64)
+        g = da.from_delayed(delcall(*tup), (size, nsamples, ncombs), float64)
         genotype.append(g)
 
     return da.concatenate(genotype)
